Package: shazam
Type: Package
Version: 1.3.2
Date: 2026-04-29
Authors@R: c(person("Mohamed", "Uduman", role=c("aut"), 
                    email="mohamed.uduman@yale.edu"),
             person("Namita", "Gupta", role=c("aut"), 
                    email="namita.gupta@yale.edu"),
             person("Susanna", "Marquez", role=c("aut","cre"), 
                    email="susanna.marquez@yale.edu"),
             person("Julian", "Zhou", role=c("aut"), 
                    email="julian.zhou@yale.edu"),
             person("Nima", "Nouri", role=c("aut"), 
                    email="nima.nouri@yale.edu"),
             person("Noah", "Yann Lee", role=c("aut"),
                    email="noah.yann.lee@yale.edu"),                    
             person("Ang", "Cui", role=c("ctb"), 
                    email="angcui@mit.edu"),
             person("Jason", "Vander Heiden", role=c("aut"), 
                    email="jason.vanderheiden@gmail.com"),
             person("Gur", "Yaari", role=c("aut"), 
                    email="gur.yaari@biu.ac.il"),
             person("Steven", "Kleinstein", role=c("aut", "cph"), 
                    email="steven.kleinstein@yale.edu"))
Title: Immunoglobulin Somatic Hypermutation Analysis
Description: Provides a computational framework for analyzing mutations in
    immunoglobulin (Ig) sequences. Includes methods for Bayesian estimation of
    antigen-driven selection pressure, mutational load quantification, building of
    somatic hypermutation (SHM) models, and model-dependent distance calculations.
    Also includes empirically derived models of SHM for both mice and humans.
    Citations: 
    Gupta and Vander Heiden, et al (2015) <doi:10.1093/bioinformatics/btv359>, 
    Yaari, et al (2012) <doi:10.1093/nar/gks457>, 
    Yaari, et al (2013) <doi:10.3389/fimmu.2013.00358>, 
    Cui, et al (2016) <doi:10.4049/jimmunol.1502263>.
License: AGPL-3
URL: http://shazam.readthedocs.io
BugReports: https://github.com/immcantation/shazam/issues
LazyData: true
BuildVignettes: true
VignetteBuilder: knitr, rmarkdown
Encoding: UTF-8
biocViews:
Depends: R (>= 4.0), ggplot2 (>= 3.4.0)
Imports: alakazam (>= 1.4.1), ape, diptest, doParallel, dplyr (>= 1.0),
        foreach, graphics, grid, igraph (>= 1.5.0), iterators,
        KernSmooth, lazyeval, MASS, methods, parallel, progress, rlang,
        scales, seqinr, stats, stringi (>= 1.1.3), tidyr, tidyselect,
        utils
Suggests: knitr, rmarkdown, testthat, tibble
Collate: 'Shazam.R' 'Core.R' 'RegionDefinitions.R' 'Baseline.R'
        'ConvertNumbering.R' 'MutationProfiling.R' 'Deprecated.R'
        'DistToNearest.R' 'MutationDefinitions.R' 'RegionsExtend.R'
        'Shmulate.R' 'TargetingModels.R' 'kedd.R'
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-30 21:27:33 UTC; root
Author: Mohamed Uduman [aut],
  Namita Gupta [aut],
  Susanna Marquez [aut, cre],
  Julian Zhou [aut],
  Nima Nouri [aut],
  Noah Yann Lee [aut],
  Ang Cui [ctb],
  Jason Vander Heiden [aut],
  Gur Yaari [aut],
  Steven Kleinstein [aut, cph]
Maintainer: Susanna Marquez <susanna.marquez@yale.edu>
Repository: CRAN
Date/Publication: 2026-05-01 07:20:02 UTC
Built: R 4.5.2; ; 2026-05-01 10:51:54 UTC; unix
